/*
 * Copyright (C) 2003 TAM(Teppei Tamra) <tam-t@par.odn.ne.jp>
 */

#include "tavia.h"
#include "taviapartsview.h"

#include <qlayout.h>
#include <qframe.h>
#include <qlabel.h>
#include <qcursor.h>
#include <qsize.h>
#include <kdockwidget.h>
#include <kaction.h>
#include <kmainwindow.h>
#include <khelpmenu.h>
#include <kstatusbar.h>
#include <kiconloader.h>
#include <kdebug.h>
#include <klocale.h>
#include <kxmlguiclient.h>
#include <kxmlguifactory.h>

tavia::tavia()
    : TaviaMdiFrm( 0, "tavia" )
{
    resizeReady = true;

    // set the shell's ui resource file
    setXMLFile("taviaui.rc");
    
    connect(this,SIGNAL(partAdd(KParts::Part* )),this,SLOT(createGUI(KParts::Part *)));
    KStdAction::quit(this,SLOT(close()),actionCollection());
    new KAction(i18n("&Dock"),"view_choose",SHIFT+CTRL+Key_T,this,SLOT(slotDockMenuPopup()),actionCollection(),"dock");
    fullScreenToggle = new KAction(i18n("Full-Screen Mode"),"window_fullscreen",SHIFT+CTRL+Key_F,this,SLOT(slotFullScreenToggle()),actionCollection(),"fullScreenToggle");
    statusBar()->setSizeGripEnabled(false);
    setStandardToolBarMenuEnabled(true);

    sidebar = new TaviaSidebar(this,"sidebar");
    sidebar->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    addToolWindow(sidebar,KDockWidget::DockLeft,this,20);

    tab = new TaviaTabBoxFrame(this,"tab");
    addToolWindow(tab,KDockWidget::DockLeft,this,20);

    manager()->setSplitterKeepSize(true);

    createGUI(0L);


    setAutoSaveSettings();

    KConfig* config = KGlobal::config();
    config->setGroup("Config");
    int cVersion = config->readNumEntry("Version",0);
    if (cVersion) {
        switch(config->readNumEntry("MDIMode",KMdi::ChildframeMode)) {
            case KMdi::ToplevelMode: {
                switchToToplevelMode();
                break;
            }
            case KMdi::ChildframeMode: {
                break;
            }
            case KMdi::TabPageMode: {
                switchToTabPageMode();
                break;
            }
            case KMdi::IDEAlMode: {
                switchToIDEAlMode();
                break;
            }
            default: {
                break;
            }
        }
        readDockConfig(config,"DockLayout");
    }
    
    else config->writeEntry("Version","1");
    
    resizeReady = false;

    emit sidebar->initSidebar();
    
    vm = new TaviaViewManager(tab->tabbox,this,this);
    factory()->addClient(vm);
    
    connect(vm,SIGNAL(activePart(KParts::Part* )),this,SLOT(createGUI(KParts::Part*)));

    //setMenuForSDIModeSysButtons(menuBar());
    setSysButtonsAtMenuPosition();
    
    setDefaultChildFrmSize(QSize(400,300));
//    vm->addPartsView(KURL("http://nop.net-p.org/"));
//    vm->addPartsView(KURL("http://tavia.net-p.org/"));
//    vm->addPartsView(KURL("http://tavia.net-p.org/"));
//    vm->addPartsView();
//    vm->setSelected((unsigned int)0);

    connect(sidebar,SIGNAL(openURL(const KURL &)),vm,SLOT(openURLwithCurrentView(const KURL &)));
    connect(sidebar,SIGNAL(openURL(const KURL &,const KParts::URLArgs &)),vm,SLOT(openURLwithCurrentView(const KURL &,const KParts::URLArgs &)));
    connect(sidebar,SIGNAL(openURLwithNewTab(const KURL &)),vm,SLOT(addPartsView(const KURL &)));
    connect(sidebar,SIGNAL(openURLwithNewTab(const KURL &,const KParts::URLArgs &)),vm,SLOT(addPartsView(const KURL &,const KParts::URLArgs &)));


}

tavia::~tavia()
{
    delete vm;
    KConfig* config = KGlobal::config();
    writeDockConfig(config,"DockLayout");
    config->setGroup("Config");
    config->writeEntry("MDIMode",mdiMode());
    //config->writeEntry("sizes",splitter->sizes());
}

/*!
    \fn tavia::bugReport()
 */
void tavia::bugReport()
{
    //emit tab->addView(KURL("http://nop.net-p.org/modules/newbb/viewforum.php?forum=6"),KParts::URLArgs());
}

/*!
    \fn tavia::slotFullScreenToggle()
 */
void tavia::slotFullScreenToggle()
{
    resizeReady = true;
    // fullscreenɡKonquerorΥɤ㤷ȡ
    if(isFullScreen()) {
        fullScreenToggle->setText(i18n("Full-Screen Mode"));
        fullScreenToggle->setIcon("window_fullscreen");
        showNormal();
    } else {
        fullScreenToggle->setText(i18n("Exit Full-Screen Mode"));
        fullScreenToggle->setIcon("window_nofullscreen");
        showFullScreen();
    }
    resizeReady = false;
}


/*!
    \fn tavia::slotDockMenuPopup()
 */
void tavia::slotDockMenuPopup()
{
    dockHideShowMenu()->popup(QCursor::pos());
}

/*!
    \fn tavia::resize(int w,int h)
 */
void tavia::resize(int w,int h)
{
    // JavaScriptˤ륦ɥꥵ̵롣
    // κǤ롣¾ˡϤʤ
    if (resizeReady) TaviaMdiFrm::resize(w,h);

    return;
}


/*!
    \fn tavia::resize(const QSize &s)
 */
void tavia::resize(const QSize &s)
{
    TaviaMdiFrm::resize(s.width(),s.height());
}




/*!
    \fn tavia::activePart(KParts::Part *part)
 */
void tavia::activePart(KParts::Part *part)
{
    createGUI(0L);
    createGUI(part);
}



/*!
    \fn tavia::close()
 */
bool tavia::close()
{
    return(TaviaMdiFrm::close());
}
