/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "taviapartsview.h"
#include <taviaviewmanager.h>
#include <taviaurclistitem.h>
#include <taviaurcdrag.h>
#include <taviaurcdropbutton.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qpushbutton.h>
#include <qcursor.h>
#include <kparts/browserrun.h>
#include <kparts/browserextension.h>
#include <kactioncollection.h>
#include <kaction.h>
#include <kxmlguiclient.h>
#include <kxmlguifactory.h>
#include <kmainwindow.h>
#include <kfileitem.h>
#include <kdebug.h>
#include <kservice.h>
#include <klibloader.h>
#include <ktrader.h>
#include <kaction.h>
#include <krun.h>
#include <kapplication.h>
#include <dcopclient.h>

TaviaPartsView::TaviaPartsView(TaviaViewManager *pm, QWidget *parent, const char *name, WFlags f)
 : TaviaView(parent, name, f)
{
    manager = pm;
    service = 0L;
    viewActive = false;
    previewjob = 0L;
    currentHistory = 0;
    run = 0L;
    hPopup = 0L;
    keepHistory = false;
    reloaded = false;
    delay = false;
    finished = true;
    history.setAutoDelete(true);
    urcObject = new TaviaURCObject(manager->getURCManager(),this);
    urcObject->setURCIcon("tavia");
    urcObject->setURCName("view");
    manager->getURCManager()->addConnection(urcObject,urcObject);
    urlBar = new TaviaURLBar(this);
    connect(urlBar,SIGNAL(enterURL(const KURL &)),this,SLOT(openURL(const KURL&)));
    connect(urcObject,SIGNAL(openURL(const KURL&, const KParts::URLArgs& )),
        this,SLOT(openURL(const KURL&, const KParts::URLArgs& )));
    connect(this,SIGNAL(request(const KURL&, const KParts::URLArgs& )),
        urcObject,SIGNAL(request(const KURL&, const KParts::URLArgs& )));
    setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);    

    urcMenu = new KPopupMenu(this);
    TaviaURCDropButton *urcButton = new TaviaURCDropButton(this);
    //urcButton->setText(i18n("URC"));
    urcButton->setIconSet(KGlobal::instance()->iconLoader()->loadIconSet("connect_established",KIcon::Panel,KIcon::SizeMedium));
    urcButton->setAutoRaise(true);
    connect(urcButton,SIGNAL(drop(QDropEvent* )),this,SLOT(urcDrop(QDropEvent* )));
    connect(urcButton,SIGNAL(clicked()),this,SLOT(urcMenuPopup()));
    QVBoxLayout *vlay = new QVBoxLayout(this);
    QHBoxLayout *toolbox = new QHBoxLayout(vlay);
    toolbox->addWidget(urcButton);
    QVBoxLayout *tv = new QVBoxLayout(toolbox);
    
    
    htmlPart = new KHTMLPart(this,0,0,0,KHTMLPart::BrowserViewGUI);
    htmlPart->setSelectable(true);
    htmlPart->setJScriptEnabled(true);
    htmlPart->setDNDEnabled(true);
    
    navi = new TaviaPartsViewNavigation(this);
    tv->addWidget(navi);
    
    connect(htmlPart,SIGNAL(setWindowCaption(const QString &)),this,SIGNAL(setViewCaption(const QString & )));
    connect(htmlPart,SIGNAL(setWindowCaption(const QString &)),this,SLOT(gotCaption(const QString & )));
    connect(htmlPart,SIGNAL(setStatusBarText(const QString &)),this,SIGNAL(setViewStatusText(const QString & )));
    connect(htmlPart->browserExtension(),SIGNAL(loadingProgress(int)),this,SLOT(progressUpdate(int )));
    connect(htmlPart,SIGNAL(completed()),this,SLOT(htmlCompleted()));
    connect(htmlPart,SIGNAL(completed()),this,SLOT(completed()));
    connect(htmlPart->browserExtension(),SIGNAL(createNewWindow(const KURL &,const KParts::URLArgs &,const KParts::WindowArgs &,KParts::ReadOnlyPart *&)),
        this,SLOT(createNewView(const KURL&, const KParts::URLArgs& ,const KParts::WindowArgs &,KParts::ReadOnlyPart *&)));
    connect(htmlPart->browserExtension(),SIGNAL(createNewWindow(const KURL &,const KParts::URLArgs &)),
        this,SLOT(createNewView(const KURL&, const KParts::URLArgs& )));
    connect(htmlPart,SIGNAL(destroyed(QObject *)),this,SLOT(closeView(QObject* )));
    frame = new QFrame(this);
    frame->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    frameLay = new QVBoxLayout(frame);
    tv->addWidget(urlBar);
    //toolbox->addLayout(tv);
    //vlay->addLayout(toolbox);
    vlay->addWidget(frame);
    part = htmlPart;
    
    new KAction(i18n("&Copy"),"copy",CTRL+Key_C,this,SLOT(copy()),actionCollection(),"copy");
    
    connectExtension(htmlPart);
    
    part->embed(frame);
    part->widget()->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    frameLay->addWidget(part->widget());
    vlay->activate();
    connect(htmlPart->browserExtension(),SIGNAL(setLocationBarURL (const QString &)),urlBar,SLOT(setEditText(const QString &)));
    connect(htmlPart->browserExtension(),SIGNAL(openURLRequest(const KURL &,const KParts::URLArgs &)),
        urcObject,SIGNAL(request(const KURL &,const KParts::URLArgs &)));
        //this,SLOT(openURL(const KURL &,const KParts::URLArgs &)));
    connect(htmlPart->browserExtension(),SIGNAL(popupMenu(const QPoint &,const KFileItemList &)),
        this,SLOT(popupMenu(const QPoint&, const KFileItemList& )));
    connect(htmlPart->browserExtension(),SIGNAL(popupMenu(KXMLGUIClient*, const QPoint&, const KFileItemList& )),
        this,SLOT(popupMenu(KXMLGUIClient*, const QPoint&, const KFileItemList& )));
    connect(htmlPart->browserExtension(),SIGNAL(popupMenu(const QPoint &, const KURL &, const QString &, mode_t)),
        this,SLOT(popupMenu(const QPoint&, const KURL&, const QString&, mode_t )));
    connect(htmlPart->browserExtension(),SIGNAL(popupMenu(KXMLGUIClient *, const QPoint &, const KURL &, const QString &, mode_t)),
        this,SLOT(popupMenu(KXMLGUIClient*, const QPoint&, const KURL&, const QString&, mode_t )));
    connect(htmlPart->browserExtension(),SIGNAL(popupMenu(KXMLGUIClient *, const QPoint &, const KFileItemList&, const KParts::URLArgs &, KParts::BrowserExtension::PopupFlags)),
        this,SLOT(popupMenu(KXMLGUIClient*, const QPoint&, const KFileItemList&, const KParts::URLArgs&, KParts::BrowserExtension::PopupFlags )));
    connect(htmlPart->browserExtension(),SIGNAL(popupMenu(KXMLGUIClient *, const QPoint &, const KURL &, const KParts::URLArgs &, KParts::BrowserExtension::PopupFlags, mode_t)),
        this,SLOT(popupMenu(KXMLGUIClient*, const QPoint&, const KURL&, const KParts::URLArgs&, KParts::BrowserExtension::PopupFlags, mode_t )));
    // Frameб
    connect(htmlPart->partManager(),SIGNAL(partAdded(KParts::Part *)),this,SLOT(addPart(KParts::Part* )));
    manager->getPartManager()->addPart(htmlPart,false);
    
}


TaviaPartsView::~TaviaPartsView()
{
}


/*!
    \fn TaviaPartsView::openURLDelay()
 */
void TaviaPartsView::openURLDelay()
{
    // 뤹ޤǤʤȤοĥϥϥ
    kdDebug() << "openURL Delay..." << endl;
    delay = true;
    if (finished || pArgs.redirectedRequest()) {
        //urlBar->setEditURL(pURL);
        pProgress = 0;
        reloaded = pReload;
        if (pURL.url().find(QString::fromLatin1("javascript:"),0,false ) == 0) {
            if (service != 0) {
                manager->getPartManager()->removePart(part);
                delete part;
                service = 0L;
                part = htmlPart;
                htmlPart->widget()->show();
                emit activate();
            }
            htmlPart->executeScript(KURL::decode_string(pURL.url().right( pURL.url().length() - 11)));
        } else {   
            run = new TaviaPartsViewRun(pURL,pArgs,pService,this,this,false,false,true);
        }
        navi->updateButtonStatus();
        delay = false;
    } else QTimer::singleShot(10,this,SLOT(openURLDelay()));
}


/*!
    \fn TaviaPartsView::openURL(const KURL &url,const KParts::URLArgs &args,KService::Ptr serviceType,bool reload)
 */
void TaviaPartsView::openURL(const KURL &url,const KParts::URLArgs &args,KService::Ptr serviceType,bool reload)
{

    kdDebug() << "OPENURL : " << url << endl;
    if (args.frameName.lower() == QString::fromLatin1("_blank")) {
        KParts::URLArgs newargs = args;
        newargs.frameName = "";
        createNewView(url,newargs);
        return;
    }

    if (url.url().isNull()) return;
    if (!finished || run) pageStop();
    urlBar->setEditURL(url);
    pURL = url;
    pArgs = args;
    pService = serviceType;
    pReload = reload;
    if (!delay) openURLDelay();
}

/*!
    \fn TaviaPartsView::openURL(const KURL &url,const KParts::URLArgs &args)
 */
void TaviaPartsView::openURL(const KURL &url,const KParts::URLArgs &args)
{
    KParts::URLArgs a = args;
    if (!a.metaData().contains("_TAVIA_SERVICE"))
        openURL(url,args,0L);
    else {
        KService::Ptr p = KService::serviceByName(a.metaData()["_TAVIA_SERVICE"]);
        a.metaData().remove("_TAVIA_SERVICE");
        openURL(url,a,p);
    }        
}

/*!
    \fn TaviaPartsView::openURL(const KURL &url)
 */
void TaviaPartsView::openURL(const KURL &url)
{
    openURL(url,KParts::URLArgs(),0L);
}


/*!
    \fn TaviaPartsView::loadingStarted()
 */
void TaviaPartsView::loadingStarted()
{
    // ɤ߹߳Ϥ
    finished = false;
    run = 0L;
    emit currentURL(pURL);
    if (!reloaded) {
        if (history.count()) currentHistory ++;
        else currentHistory = 0;
        
        if (history.count()) {
            for(unsigned i = (history.count() - 1);i >= currentHistory;i --) {
                history.remove(i);
            }
        }
        //TaviaCacheEntry *h = new TaviaHistoryEntry;
        //history.append(new TaviaHistoryEntry);
    }
    htmlThumbnail.resize(400,400);
    htmlThumbnail.fill(QColor(245,245,245));;
    emit thumbnailReady(htmlThumbnail);

}


/*!
    \fn TaviaPartsView::gotCaption(const QString &c)
 */
void TaviaPartsView::gotCaption(const QString &c)
{
    caption = c;
    urcObject->setURCName(i18n("View:") + c);
}



/*!
    \fn TaviaPartsView::popupMenu (const QPoint &global, const KFileItemList &items)
 */
void TaviaPartsView::popupMenu (const QPoint &global, const KFileItemList &items)
{
    kdDebug() << "popup:::1" << endl;
    popupMenu(0L,global,items);
}


/*!
    \fn TaviaPartsView::popupMenu (KXMLGUIClient *client, const QPoint &global, const KFileItemList &items)
 */
void TaviaPartsView::popupMenu (KXMLGUIClient *client, const QPoint &global, const KFileItemList &items)
{
    kdDebug() << "popup:::2" << endl;
    popupMenu(client,global,items,KParts::URLArgs(),KParts::BrowserExtension::DefaultPopupItems,true);
}


/*!
    \fn TaviaPartsView::popupMenu (const QPoint &global, const KURL &url, const QString &mimeType, mode_t mode=(mode_t)-1)
 */
void TaviaPartsView::popupMenu (const QPoint &global, const KURL &url, const QString &mimeType, mode_t mode)
{
    kdDebug() << "popup:::3" << endl;
    popupMenu(0L,global,url,mimeType,mode);
}


/*!
    \fn TaviaPartsView::popupMenu (KXMLGUIClient *client, const QPoint &global, const KURL &url, const QString &mimeType, mode_t mode=(mode_t)-1)
 */
void TaviaPartsView::popupMenu (KXMLGUIClient *client, const QPoint &global, const KURL &url, const QString &mimeType, mode_t mode)
{
    kdDebug() << "popup:::4" << endl;
    KFileItem item(url,mimeType,mode);
    KFileItemList items;
    items.append(&item);
    popupMenu(client,global,items);
}


/*!
    \fn TaviaPartsView::popupMenu (KXMLGUIClient *client, const QPoint &global, const KURL &url, const KParts::URLArgs &args, KParts::BrowserExtension::PopupFlags i, mode_t mode=(mode_t)-1)
 */
void TaviaPartsView::popupMenu (KXMLGUIClient *client, const QPoint &global, const KURL &url, const KParts::URLArgs &args, KParts::BrowserExtension::PopupFlags i, mode_t mode)
{
    kdDebug() << "popup:::5" << endl;
    KFileItem item(url,args.serviceType,mode);
    KFileItemList items;
    items.append(&item);
    popupMenu(client,global,items,args,i,true);
}


/*!
    \fn TaviaPartsView::popupMenu (KXMLGUIClient *client, const QPoint &global, const KFileItemList &items, const KParts::URLArgs &args, KParts::BrowserExtension::PopupFlags i)
 */
void TaviaPartsView::popupMenu (KXMLGUIClient *client, const QPoint &global, const KFileItemList &items, const KParts::URLArgs &args, KParts::BrowserExtension::PopupFlags i)
{
    kdDebug() << "popup:::6" << endl;
    popupMenu(client,global,items,args,i,true);
}


/*!
    \fn TaviaPartsView::popupMenu (KXMLGUIClient *client, const QPoint &global, const KFileItemList &items, const KParts::URLArgs &args, KParts::BrowserExtension::PopupFlags i, mode_t mode=(mode_t)-1)
 */
void TaviaPartsView::popupMenu (KXMLGUIClient *client, const QPoint &global, const KFileItemList &items, const KParts::URLArgs &args, KParts::BrowserExtension::PopupFlags i, mode_t mode)
{
    kdDebug() << "popup:::MAIN" << endl;
    QMap<int,KService::Ptr> previewMap;
    QMap<int,KService::Ptr> applicationMap;
    KPopupMenu *popup = new KPopupMenu(this);
    previewMap.clear();
    applicationMap.clear();
    
    kdDebug() << "itemCount: " << items.count() << endl;
    
    selectedURL = items.getFirst()->url();
    kdDebug() << "itemURL: " << items.getFirst()->url() << endl;
    if (items.count() == 1) {
        KTrader::OfferList offers = KTrader::self()->query(items.getFirst()->mimetype(),
            "KParts/ReadOnlyPart",
            "(not exist [X-KDE-BrowserView-HideFromMenus] or not [X-KDE-BrowserView-HideFromMenus]) "
            "and exist [Library]",
            QString::null);
        if (items.getFirst()->url().url() == part->url().url()) popup->insertTitle(i18n("Current Page"));
        else popup->insertTitle(items.getFirst()->name());
        for(unsigned int i = 0;i < offers.count();i ++) {
            KService::Ptr ptr = offers[i];
            int pn = popup->insertItem(
                KGlobal::instance()->iconLoader()->loadIcon(offers[i]->icon(),KIcon::Small,KIcon::SizeSmall),
                offers[i]->name()
            );
            previewMap[pn] = ptr;
        }
        offers = KTrader::self()->query(items.getFirst()->mimetype(), "Type == 'Application'");
        if (offers.count() != 0) popup->insertTitle(i18n("Application"));
        for(unsigned int i = 0;i < offers.count();i ++) {
            KService::Ptr ptr = offers[i];
            int pn = popup->insertItem(
                KGlobal::instance()->iconLoader()->loadIcon(offers[i]->icon(),KIcon::Small,KIcon::SizeSmall),
                offers[i]->name()
            );
            applicationMap[pn] = ptr;
        }

    }
    if (client != 0) {
        popup->insertTitle(i18n("Actions"));
        for(unsigned int i = 0;i < client->actionCollection()->count();i ++) {
            client->actionCollection()->action(i)->plug(popup);
        }
    }
    //popup->popup(global);
    int p = popup->exec(global);
    if (p != -1) {
        if (previewMap.contains(p)) {
            KParts::URLArgs args;
            args.metaData()["_TAVIA_SERVICE"] = previewMap[p]->name();
            emit request(selectedURL,args);
        }
        if (applicationMap.contains(p)) {
            KURL::List l;
            l.append(selectedURL);
            KRun::run(*applicationMap[p],l);
        }
    }
    delete popup;

}


/*!
    \fn TaviaPartsView::*getGUIClient()
 */
KXMLGUIClient * TaviaPartsView::getGUIClient()
{
    return(part);
}



/*!
    \fn TaviaPartsView::connectExtension( KParts::BrowserExtension *ext )
    from Konqueror.
    ShellΥPartΥ³ˡ
    @todo disconnectѰդ뤳ȡ
 */
void TaviaPartsView::connectExtension( KParts::ReadOnlyPart *p)
{
    KParts::BrowserExtension *ext = KParts::BrowserExtension::childObject(p);
    if (ext == 0L) return;
    KParts::BrowserExtension::ActionSlotMap * actionSlotMap = KParts::BrowserExtension::actionSlotMapPtr();
    KParts::BrowserExtension::ActionSlotMap::ConstIterator it = actionSlotMap->begin();
    KParts::BrowserExtension::ActionSlotMap::ConstIterator itEnd = actionSlotMap->end();

    QStrList slotNames = ext->metaObject()->slotNames();
    
    // debug
    for(unsigned int i = 0;i < slotNames.count();i ++) {
        kdDebug() << "SLOTS " << i << ": " << slotNames.at(i) << endl;
    }
    
    
    for ( ; it != itEnd ; ++it ) {
        KAction * act = actionCollection()->action( it.key() );
        if ( act ) {
            if ( slotNames.contains( it.key()+"()" ) ) {
                connect( act, SIGNAL( activated() ), ext, it.data() /* SLOT(slot name) */ );
                act->setEnabled( ext->isActionEnabled( it.key() ) );
            } else act->setEnabled(false);
        } else kdError(1202) << "Error in BrowserExtension::actionSlotMap(), unknown action : " << it.key() << endl;
    }
}



/*!
    \fn TaviaPartsView::copy()
 */
void TaviaPartsView::copy()
{
    kdDebug() << "!!!COPY!!!" << endl;
}


/*!
    \fn TaviaPartsView::activate()
 */
void TaviaPartsView::activate()
{
    kdDebug() << "TaviaPartsView: " << this << " Activated!" << endl;
    if (manager->isSelected(this)) {
        viewActive = true;
        if (manager->getPartManager()->activePart() != part) manager->getPartManager()->setActivePart(part);
    }
}

/*!
    \fn TaviaPartsView::deactivate()
 */
void TaviaPartsView::deactivate()
{
    kdDebug() << "TaviaPartsView: " << this << " Deactivated!" << endl;
    viewActive = false;
}


/*!
    \fn TaviaPartsView::createHtmlThumbnail()
 */
void TaviaPartsView::createHtmlThumbnail()
{
    QPixmap pix;
    pix.resize(400,400);
    pix.fill(QColor(245,245,245));
    QRect rc(0,0,400,400);
    QPainter p;
    p.begin(&pix);
    htmlPart->paint(&p, rc,1);
    p.end();
    htmlThumbnail = pix;
    kdDebug() << "thumbnail updated." << endl;
}



/*!
    \fn TaviaPartsView::getPreviewStart()
 */
void TaviaPartsView::getPreviewStart()
{
    kdDebug() << "previewjob start." << endl;
    KFileItemList l;
    KFileItem *i = new KFileItem(part->url(),"",0);
    l.append(i);
    previewjob = new KIO::PreviewJob(l,400,400,100,0,true,false,0);
    connect(previewjob,SIGNAL(gotPreview(const KFileItem*, const QPixmap& )),this,SLOT(gotPreview(const KFileItem*, const QPixmap& )));
    connect(previewjob,SIGNAL(failed(const KFileItem*)),this,SLOT(gotPreviewFailed(const KFileItem*)));
}



/*!
    \fn TaviaPartsView::gotPreview(const KFileItem* item, const QPixmap& pixmap)
 */
void TaviaPartsView::gotPreview(const KFileItem* item, const QPixmap& pixmap)
{
    //KFileItem i = *item;
    if (item->url() != part->url()) return;
    QPixmap pix;
    pix.resize(400,400);
    pix.fill(QColor(245,245,245));
    QRect rc(0,0,pixmap.width(),pixmap.height());
    QPainter p;
    p.begin(&pix);
    p.drawPixmap(rc,pixmap);
    p.end();
    htmlThumbnail = pix;
    
    
    emit thumbnailReady(htmlThumbnail);
    if (history.at(currentHistory)->url == item->url()) history.at(currentHistory)->thumbnail = pix;
    kdDebug() << "thumbnail updated.(previewjob)" << endl;
}


/*!
    \fn TaviaPartsView::gotPreviewFailed(const KFileItem *item)
 */
void TaviaPartsView::gotPreviewFailed(const KFileItem *item)
{
    if (item->url() != part->url()) return;
    QPixmap pix;
    QPixmap pixmap = item->pixmap(300);
    pix.resize(400,400);
    pix.fill(QColor(245,245,245));
    QRect rc(0,0,pixmap.width(),pixmap.height());
    QPainter p;
    p.begin(&pix);
    p.drawPixmap(rc,pixmap);
    p.end();
    htmlThumbnail = pix;
    
    emit thumbnailReady(htmlThumbnail);
    if (history.at(currentHistory)->url == item->url()) history.at(currentHistory)->thumbnail = pix;
    kdDebug() << "thumbnail updated.(previewjob failed)" << endl;
    
}


/*!
    \fn TaviaPartsView::htmlCompleted()
 */
void TaviaPartsView::htmlCompleted()
{

    createHtmlThumbnail();
    emit thumbnailReady(htmlThumbnail);
    
}

/*!
    \fn TaviaPartsView::completed()
 */
void TaviaPartsView::completed()
{
    completed(false);
}

/*!
    \fn TaviaPartsView::completed(bool p)
 */
void TaviaPartsView::completed(bool p)
{
    QByteArray data;
    QDataStream str(data, IO_WriteOnly);
    str << htmlPart->url();
    dynamic_cast<KApplication*>(KGlobal::instance())->dcopClient()->send("kded","favicons", "downloadHostIcon(KURL)", data);
    KURL u;
    if (part->url().url().at(0) == '/') u.setPath(part->url().url());
    else u = part->url().url();
    QString icon = KMimeType::iconForURL(u);
    kdDebug() << "favicon : " << icon << endl;
    
    if (!keepHistory) {
        saveHistory(currentHistory);
    }


    
    TaviaPartsView::icon = KGlobal::instance()->iconLoader()->loadIcon(icon,KIcon::Small,KIcon::SizeSmall);
    emit setViewIcon(TaviaPartsView::icon);
    urcObject->setURCIcon(icon);
    
    kdDebug() << "History: " << currentHistory << "/" << history.count() << endl;
    keepHistory = false;
    reloaded = false;
    finished = true;
    navi->updateButtonStatus();
    urlBar->addURLHistory(part->url());
}


/*!
    \fn TaviaPartsView::urcMenuPopup()
 */
void TaviaPartsView::urcMenuPopup()
{
    urcMenu->clear();
    
    QPtrList<TaviaURCObject> l = urcObject->getManager()->getConnectionList(urcObject);
    if (l.count()) urcMenu->insertTitle(i18n("Connection List"));
    else urcMenu->insertTitle(i18n("no Connection"));
    for(unsigned int i = 0;i < l.count();i ++) {
        TaviaURCListItem *li = new TaviaURCListItem(l.at(i),urcMenu);
        if (l.at(i) == urcObject) li->setLabelText(i18n("self"));
        else li->setLabelText(l.at(i)->getURCName());
        li->setIcon(l.at(i)->getURCIcon());
        urcMenu->insertItem(li);
        connect(li,SIGNAL(dragStart(TaviaURCObject* )),this,SLOT(urcDrag(TaviaURCObject* )));
        connect(li,SIGNAL(dcButtonClicked(TaviaURCObject* )),this,SLOT(urcDcButtonClicked(TaviaURCObject* )));
    }
    TaviaURCListItem *li = new TaviaURCListItem(0L,urcMenu);
    li->setLabelText(i18n("New Connection"));
    urcMenu->insertItem(li);
    connect(li,SIGNAL(dragStart(TaviaURCObject* )),this,SLOT(urcDrag(TaviaURCObject* )));
    
    urcMenu->popup(QCursor::pos());
}



/*!
    \fn TaviaPartsView::urcDrag(TaviaURCObject *o)
 */
void TaviaPartsView::urcDrag(TaviaURCObject *o)
{
    // ɬhide뤳ȡ
    // @todo ꡼ǽͭꡣ
    urcMenu->hide();
    urcMenu->clear();
    QDragObject *d = new TaviaURCDrag(urcObject->getManager(),urcObject,o,this);
    d->dragCopy();
}


/*!
    \fn TaviaPartsView::urcDrop(QDropEvent *e)
 */
void TaviaPartsView::urcDrop(QDropEvent *e)
{
    TaviaURCObject *from = 0L,*old = 0L;
    TaviaURCDrag::decode(e,urcObject->getManager(),from,old);
    kdDebug() << "URCDrop...from:" << from << " old:" << old  << endl;
    if (old) urcObject->getManager()->delConnection(from,old);
    urcObject->getManager()->addConnection(from,urcObject);
}


/*!
    \fn TaviaPartsView::urcDcButtonClicked(TaviaURCObject *to)
 */
void TaviaPartsView::urcDcButtonClicked(TaviaURCObject *to)
{
    // @todo ꡼ǽͭꡣ
    urcMenu->hide();
    urcMenu->clear();
    urcObject->getManager()->delConnection(urcObject,to);
}


/*!
    \fn TaviaPartsView::addPart(KParts::Part *part)
 */
void TaviaPartsView::addPart(KParts::Part *part)
{
    // @todo ե졼б
    // ʤʡ
    /*
    KParts::BrowserExtension *ext = KParts::BrowserExtension::childObject(part);
    kdDebug() << "BrowserExtension: " << ext << endl;
    if (ext != 0L) {
        connectExtension(static_cast<KParts::ReadOnlyPart *>(part));
        //connect(ext,SIGNAL(setLocationBarURL (const QString &)),urlBar,SLOT(setEditText(const QString &)));
        disconnect(ext,SIGNAL(openURLRequest(const KURL &,const KParts::URLArgs &)));
        connect(ext,SIGNAL(openURLRequest(const KURL &,const KParts::URLArgs &)),
            urcObject,SIGNAL(request(const KURL &,const KParts::URLArgs &)));
            //this,SLOT(openURL(const KURL &,const KParts::URLArgs &)));    
        connect(ext,SIGNAL(popupMenu(const QPoint &,const KFileItemList &)),
            this,SLOT(popupMenu(const QPoint&, const KFileItemList& )));
        connect(ext,SIGNAL(popupMenu(KXMLGUIClient*, const QPoint&, const KFileItemList& )),
            this,SLOT(popupMenu(KXMLGUIClient*, const QPoint&, const KFileItemList& )));
        connect(ext,SIGNAL(popupMenu(const QPoint &, const KURL &, const QString &, mode_t)),
            this,SLOT(popupMenu(const QPoint&, const KURL&, const QString&, mode_t )));
        connect(ext,SIGNAL(popupMenu(KXMLGUIClient *, const QPoint &, const KURL &, const QString &, mode_t)),
            this,SLOT(popupMenu(KXMLGUIClient*, const QPoint&, const KURL&, const QString&, mode_t )));
        connect(ext,SIGNAL(popupMenu(KXMLGUIClient *, const QPoint &, const KFileItemList&, const KParts::URLArgs &, KParts::BrowserExtension::PopupFlags)),
            this,SLOT(popupMenu(KXMLGUIClient*, const QPoint&, const KFileItemList&, const KParts::URLArgs&, KParts::BrowserExtension::PopupFlags )));
        connect(ext,SIGNAL(popupMenu(KXMLGUIClient *, const QPoint &, const KURL &, const KParts::URLArgs &, KParts::BrowserExtension::PopupFlags, mode_t)),
            this,SLOT(popupMenu(KXMLGUIClient*, const QPoint&, const KURL&, const KParts::URLArgs&, KParts::BrowserExtension::PopupFlags, mode_t )));
    }
    */
}





/*!
    \fn TaviaPartsView::loadParts(KService::Ptr ptr)
 */
KParts::ReadOnlyPart * TaviaPartsView::loadParts(KService::Ptr ptr)
{
    KLibFactory *factory = KLibLoader::self()->factory(ptr->library());
    if (factory) {
        KParts::ReadOnlyPart *part = static_cast<KParts::ReadOnlyPart *>(factory->create(this, ptr->name(), "KParts::ReadOnlyPart"));
        service = ptr;
        connect(part,SIGNAL(setWindowCaption(const QString &)),this,SIGNAL(setViewCaption(const QString & )));
        connect(part,SIGNAL(setStatusBarText(const QString &)),this,SIGNAL(setViewStatusText(const QString & )));
        connect(part,SIGNAL(setWindowCaption(const QString &)),this,SLOT(gotCaption(const QString & )));
        connect(part,SIGNAL(completed()),this,SLOT(getPreviewStart()));
        connect(part,SIGNAL(completed()),this,SLOT(completed()));
        //connect(part,SIGNAL(canceled(const QString &)),this,SLOT(completed()));
        connect(part,SIGNAL(completed(bool)),this,SLOT(completed(bool)));
        
        KParts::BrowserExtension *ext = KParts::BrowserExtension::childObject(part);
        kdDebug() << "BrowserExtension: " << ext << endl;
        if (ext != 0L) {
            connectExtension(part);
            connect(ext,SIGNAL(setLocationBarURL (const QString &)),urlBar,SLOT(setEditText(const QString &)));
            connect(ext,SIGNAL(openURLRequest(const KURL &,const KParts::URLArgs &)),
                urcObject,SIGNAL(request(const KURL &,const KParts::URLArgs &)));
            connect(ext,SIGNAL(popupMenu(const QPoint &,const KFileItemList &)),
                this,SLOT(popupMenu(const QPoint&, const KFileItemList& )));
            connect(ext,SIGNAL(popupMenu(KXMLGUIClient*, const QPoint&, const KFileItemList& )),
                this,SLOT(popupMenu(KXMLGUIClient*, const QPoint&, const KFileItemList& )));
            connect(ext,SIGNAL(popupMenu(const QPoint &, const KURL &, const QString &, mode_t)),
                this,SLOT(popupMenu(const QPoint&, const KURL&, const QString&, mode_t )));
            connect(ext,SIGNAL(popupMenu(KXMLGUIClient *, const QPoint &, const KURL &, const QString &, mode_t)),
                this,SLOT(popupMenu(KXMLGUIClient*, const QPoint&, const KURL&, const QString&, mode_t )));
            connect(ext,SIGNAL(popupMenu(KXMLGUIClient *, const QPoint &, const KFileItemList&, const KParts::URLArgs &, KParts::BrowserExtension::PopupFlags)),
                this,SLOT(popupMenu(KXMLGUIClient*, const QPoint&, const KFileItemList&, const KParts::URLArgs&, KParts::BrowserExtension::PopupFlags )));
            connect(ext,SIGNAL(popupMenu(KXMLGUIClient *, const QPoint &, const KURL &, const KParts::URLArgs &, KParts::BrowserExtension::PopupFlags, mode_t)),
                this,SLOT(popupMenu(KXMLGUIClient*, const QPoint&, const KURL&, const KParts::URLArgs&, KParts::BrowserExtension::PopupFlags, mode_t )));
            connect(ext,SIGNAL(loadingProgress(int)),this,SLOT(progressUpdate(int )));
        }
    return(part);
    } else return(0L);
}




/*!
    \fn TaviaPartsView::restoreHistory(unsigned int p)
 */
void TaviaPartsView::restoreHistory(unsigned int p)
{
    if (history.count() == 0) return;
    if (history.count() <= p) return;
    kdDebug() << "!!! RESTORE : " << currentHistory << " !!!" << endl;
    currentHistory = p;
    if (history.at(p)->service == 0) {
        if (service != 0) {
            manager->getPartManager()->removePart(part);
            delete part;
            service = 0L;
            part = htmlPart;
            htmlPart->widget()->show();
            emit activate();
        }
        keepHistory = true;
        QDataStream s(history.at(p)->state, IO_ReadOnly);
        htmlPart->browserExtension()->restoreState(s);
        urlBar->setEditURL(history.at(p)->url);
    } else {
        reload();
    }
}


/*!
    \fn TaviaPartsView::saveHistory(unsigned int p)
 */
void TaviaPartsView::saveHistory(unsigned int p)
{
    TaviaHistoryEntry *h;
    h = history.at(p);
    if (!h) {
        h = new TaviaHistoryEntry;
        history.append(h);
    }
    h->state.resize(0);
    h->service = service;
    h->url = part->url();
    if (part != htmlPart) {
        h->noCache = true;
    } else {
        h->noCache = false;
        QDataStream s(h->state,IO_WriteOnly);
        htmlPart->browserExtension()->saveState(s);
    }
    KParts::BrowserExtension *ext = KParts::BrowserExtension::childObject(part);
    if (ext) {
        h->args = ext->urlArgs();
    } else {
        h->args = KParts::URLArgs();
    }
    h->caption = caption;
    h->thumbnail = htmlThumbnail;
    
    for(unsigned i = 0;i < history.count();i ++) {
        kdDebug() << "HHH History: " << i << " URL : " << history.at(i)->url << " noCache :" << history.at(i)->noCache << endl;
    }
}


/*!
    \fn TaviaPartsView::reload()
 */
void TaviaPartsView::reload()
{
    if (history.count() == 0) return;
    //reloaded = true;
    //part->closeURL();
    openURL(history.at(currentHistory)->url,history.at(currentHistory)->args,history.at(currentHistory)->service,true);
}




/*!
    \fn TaviaPartsView::pageBack()
 */
void TaviaPartsView::pageBack()
{
    pageStop();
    restoreHistory(currentHistory - 1);
}


/*!
    \fn TaviaPartsView::pageForward()
 */
void TaviaPartsView::pageForward()
{
    pageStop();
    if ((currentHistory + 1) == history.count()) return;
    restoreHistory(currentHistory + 1);
}


/*!
    \fn TaviaPartsView::pageReload()
 */
void TaviaPartsView::pageReload()
{
    pageStop();
    reload();
}


/*!
    \fn TaviaPartsView::pageStop()
 */
void TaviaPartsView::pageStop()
{
    part->closeURL();
    if (run != 0) {
        run->abort();
        run = 0L;
    } else {
        //if (part != htmlPart) completed();
        completed();
    }

}




/*!
    \fn TaviaPartsView::progressUpdate(int p)
 */
void TaviaPartsView::progressUpdate(int p)
{
    kdDebug() << "PPP progress: " << p << endl;
    
    if (p == 100) return;
    if (p <= pProgress) return;
    pProgress = p;
    kdDebug() << "progress:: " << p << endl;
    createHtmlThumbnail();
    //
    // вߥ!!
    //
    QPixmap pix = htmlThumbnail;
    QPainter paint;
    QPen pen(Qt::red,2);
    QBrush b(Qt::red,Qt::Dense6Pattern);
    paint.begin(&pix);
    paint.setPen(pen);
    paint.drawEllipse(20,20,200,160);
    paint.drawArc(20,20,200,170,-180*16,180*16);
    paint.setBrush(b);
    paint.drawPie(20,20,200,160,(90 * 16) - ((360 * 16 * p) / 100),(360 * 16 * p) / 100);
    kdDebug() << "Pie: " << (360 * 16 * 100) / p << endl;
    paint.end();
    progressPixmap = pix;
    thumbnailReady(progressPixmap);
}


/*!
    \fn TaviaPartsView::createNewView(const KURL &url,const KParts::URLArgs &args,const KParts::WindowArgs &wargs,KParts::ReadOnlyPart *&p)
 */
void TaviaPartsView::createNewView(const KURL &url,const KParts::URLArgs &args,const KParts::WindowArgs &wargs,KParts::ReadOnlyPart *&p)
{
    kdDebug() << "CreateNewView URL: " << url << endl;
    KParts::URLArgs newargs = args;
    if (args.frameName.lower() == QString::fromLatin1("_blank")) {
        newargs.frameName = "";
    }
    manager->addPartsView(url,newargs,wargs,p);
}


/*!
    \fn TaviaPartsView::createNewView(const KURL &url,const KParts::URLArgs &args)
 */
void TaviaPartsView::createNewView(const KURL &url,const KParts::URLArgs &args)
{
    kdDebug() << "CreateNewView URL: " << url << endl;
    KParts::URLArgs newargs = args;
    if (args.frameName.lower() == QString::fromLatin1("_blank")) {
        newargs.frameName = "";
    }
    manager->addPartsView(url,newargs);
}


/*!
    \fn TaviaPartsView::closeView(QObject *o)
 */
void TaviaPartsView::closeView(QObject *o)
{
    if (run) run->abort();
    manager->closeView(this);
}



/*!
    \fn TaviaPartsView::historyPopup()
 */
void TaviaPartsView::historyPopup()
{
    if (hPopup) return;
    hPopup = new TaviaPartsViewHistoryPopup(this,"",Qt::WType_Popup);// | Qt::WX11BypassWM | Qt::WShowModal);
    connect(hPopup,SIGNAL(historyEnter(int)),this,SLOT(historyPopupEnter(int )));
}


/*!
    \fn TaviaPartsView::historyPopupEnter(int num)
 */
void TaviaPartsView::historyPopupEnter(int num)
{
    kdDebug() << "HistoryPopup Enter: " << num << endl;
    delete hPopup;
    hPopup = 0L;
    if ((num == -1) || ((unsigned int)num == currentHistory)) return;
    pageStop();
    restoreHistory(num);
}

