/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef TAVIAPARTSVIEW_H
#define TAVIAPARTSVIEW_H

#include <taviaview.h>
#include <taviaviewmanager.h>
#include <taviaurlbar.h>
#include <taviapartsviewrun.h>
#include <taviaurcobject.h>
#include <taviapartsviewnavigation.h>
#include <taviapartsviewhistorypopup.h>
#include <kparts/partmanager.h>
#include <kparts/part.h>
#include <khtml_part.h>
#include <kpopupmenu.h>
#include <kservice.h>
#include <qframe.h>
#include <qlayout.h>
#include <qmap.h>
#include <kio/previewjob.h>


/**
@author TAM(Teppei Tamra)
*/
class TaviaPartsViewRun;
class TaviaPartsViewNavigation;
class TaviaPartsViewHistoryPopup;

class TaviaHistoryEntry{
public:
    KService::Ptr service;
    KURL url;
    KParts::URLArgs args;
    QByteArray state;
    bool noCache;
    QPixmap thumbnail;
    QString caption;
};

class TaviaPartsView : public TaviaView
{
Q_OBJECT
public:
    friend class TaviaPartsViewRun;
    friend class TaviaPartsViewHistoryPopup;
    friend class TaviaPartsViewNavigation;
    friend class TaviaViewManager;
    
    TaviaPartsView(TaviaViewManager *pm, QWidget *parent = 0, const char *name = 0, WFlags f = 0);

    ~TaviaPartsView();
    virtual KXMLGUIClient * getGUIClient();

public slots:
    virtual void openURL(const KURL &url,const KParts::URLArgs &args,KService::Ptr serviceType,bool reload = false);
    virtual void openURL(const KURL &url,const KParts::URLArgs &args);
    virtual void openURL(const KURL &url);
    virtual void activate();
    virtual void deactivate();
protected:
    TaviaViewManager *manager;
    TaviaURCObject *urcObject;
    TaviaPartsViewRun *run;
    TaviaPartsViewNavigation *navi;
    TaviaPartsViewHistoryPopup *hPopup;
    KHTMLPart *htmlPart;
    KParts::ReadOnlyPart *part;
    TaviaURLBar *urlBar;
    QFrame *frame;
    QVBoxLayout *frameLay;
    KService::Ptr service;
    KURL selectedURL;
    QPixmap htmlThumbnail;
    QPixmap progressPixmap;
    QPixmap icon;
    bool viewActive;
    KPopupMenu *urcMenu;
    QPtrList<TaviaHistoryEntry> history;
    unsigned int currentHistory;
    KIO::PreviewJob *previewjob;
    bool keepHistory;
    bool reloaded;
    bool finished;
    bool delay;
    QString caption;
    KURL pURL;
    KParts::URLArgs pArgs;
    KService::Ptr pService;
    bool pReload;
    int pProgress;
protected:
    void connectExtension( KParts::ReadOnlyPart *p );
    KParts::ReadOnlyPart * loadParts(KService::Ptr ptr);
    void restoreHistory(unsigned int p);
    void saveHistory(unsigned int p);
    void reload();
    void createHtmlThumbnail();
    void loadingStarted();

protected slots:
    void popupMenu (const QPoint &global, const KFileItemList &items);
    void popupMenu (KXMLGUIClient *client, const QPoint &global, const KFileItemList &items);
    void popupMenu (const QPoint &global, const KURL &url, const QString &mimeType, mode_t mode);
    void popupMenu (KXMLGUIClient *client, const QPoint &global, const KURL &url, const QString &mimeType, mode_t mode);
    void popupMenu (KXMLGUIClient *client, const QPoint &global, const KURL &url, const KParts::URLArgs &args, KParts::BrowserExtension::PopupFlags i, mode_t mode);
    void popupMenu (KXMLGUIClient *client, const QPoint &global, const KFileItemList &items, const KParts::URLArgs &args, KParts::BrowserExtension::PopupFlags i);
    void popupMenu (KXMLGUIClient *client, const QPoint &global, const KFileItemList &items, const KParts::URLArgs &args, KParts::BrowserExtension::PopupFlags i, mode_t mode);
    void copy();
    void htmlCompleted();
    void urcMenuPopup();
    void urcDrag(TaviaURCObject *o);
    void urcDrop(QDropEvent *e);
    void urcDcButtonClicked(TaviaURCObject *to);
    void addPart(KParts::Part *part);
    void gotPreview(const KFileItem* item, const QPixmap& pixmap);
    void getPreviewStart();
    void gotCaption(const QString &c);
    void completed();
    void completed(bool p);
    void pageBack();
    void pageForward();
    void pageReload();
    void pageStop();
    void openURLDelay();
signals:
    void activePart(KParts::Part *part);
    void request(const KURL &url,const KParts::URLArgs &args);
protected slots:
    void progressUpdate(int p);
    void createNewView(const KURL &url,const KParts::URLArgs &args,const KParts::WindowArgs &wargs,KParts::ReadOnlyPart *&p);
    void createNewView(const KURL &url,const KParts::URLArgs &args);
    void closeView(QObject *o);
    void gotPreviewFailed(const KFileItem *item);
    void historyPopup();
    void historyPopupEnter(int num);
};

#endif
