/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "taviapartsviewnavigation.h"
#include <kiconloader.h>
#include <kstandarddirs.h>
#include <kglobal.h>
#include <qcursor.h>

TaviaPartsViewNavigation::TaviaPartsViewNavigation(TaviaPartsView *parent, const char *name)
 : QWidget(parent, name) , KBookmarkOwner() , KXMLGUIClient()
{
    view = parent;
    QVBoxLayout *vlay = new QVBoxLayout(this);
    //toolBar = new KToolBar(view->manager->getMdiFrm(),this,true,"viewToolBar",true,true),
    toolBar = new KToolBar(this,"viewToolBar",true,true),
    toolBar->applySettings(KGlobal::config(),"viewToolbarSetting");
    vlay->addWidget(toolBar);
    KBookmarkManager *bookmarkManager;
    bookmarkManager = KBookmarkManager::managerForFile(locateLocal("data","tavia/bookmarks.xml"));
    bookmarkManager->setShowNSBookmarks(false);
    bookmarkMenu = new KPopupMenu(this);
    new KBookmarkMenu(bookmarkManager,this,bookmarkMenu,0L,true);
    
    back = new KAction(i18n("Back"),"back",ALT+Key_Left,view,SLOT(pageBack()),actionCollection(),"viewBack");
    hstr = new KAction(i18n("History"),"history",ALT+Key_H,view,SLOT(historyPopup()),actionCollection(),"viewHistory");
    forw = new KAction(i18n("Forward"),"forward",ALT+Key_Right,view,SLOT(pageForward()),actionCollection(),"viewForward");
    reld = new KAction(i18n("Reload"),"reload",Key_F5,view,SLOT(pageReload()),actionCollection(),"viewReload");
    stop = new KAction(i18n("Stop"),"stop",Key_Escape,view,SLOT(pageStop()),actionCollection(),"viewStop");
    bkmk = new KAction(i18n("Bookmarks"),"bookmark",ALT+Key_B,this,SLOT(bookmarkPopup()),actionCollection(),"viewBookmark");
        reld->setEnabled(false);
        hstr->setEnabled(false);
        back->setEnabled(false);
        forw->setEnabled(false);
        stop->setEnabled(false);
    back->plug(toolBar);
    hstr->plug(toolBar);
    forw->plug(toolBar);
    reld->plug(toolBar);
    stop->plug(toolBar);
    bkmk->plug(toolBar);
    
}


TaviaPartsViewNavigation::~TaviaPartsViewNavigation()
{
    toolBar->saveSettings(KGlobal::config(),"viewToolbarSetting");
    delete(toolBar);
}




/*!
    \fn TaviaPartsViewNavigation::updateButtonStatus()
 */
void TaviaPartsViewNavigation::updateButtonStatus()
{
    if (!view->history.count()) {
        reld->setEnabled(false);
        hstr->setEnabled(false);
        back->setEnabled(false);
        forw->setEnabled(false);
    } else {
        reld->setEnabled(true);
        hstr->setEnabled(true);
        
        if (view->history.count() == 1) {
            back->setEnabled(false);
            forw->setEnabled(false);
        } else if (view->currentHistory == 0) {
            back->setEnabled(false);
            forw->setEnabled(true);
        } else if (view->currentHistory == (view->history.count() - 1)) {
            back->setEnabled(true);
            forw->setEnabled(false);
        } else {
        reld->setEnabled(true);
            back->setEnabled(true);
            forw->setEnabled(true);
        }
    }
    if ((!view->finished) || view->run) {
        stop->setEnabled(true);
        reld->setEnabled(false);
    } else {
        stop->setEnabled(false);
        reld->setEnabled(true);
    }            
}


/*!
    \fn TaviaPartsViewNavigation::bookmarkPopup()
 */
void TaviaPartsViewNavigation::bookmarkPopup()
{
    bookmarkMenu->popup(QCursor::pos());
}


/*!
    \fn TaviaPartsViewNavigation::openBookmarkURL(const QString &url)
 */
void TaviaPartsViewNavigation::openBookmarkURL(const QString &url)
{
    view->openURL(KURL(url));
}


/*!
    \fn TaviaPartsViewNavigation::currentTitle() const
 */
QString TaviaPartsViewNavigation::currentTitle() const
{
    return(view->caption);
}


/*!
    \fn TaviaPartsViewNavigation::currentURL() const
 */
QString TaviaPartsViewNavigation::currentURL() const
{
    return(view->part->url().url());
}
