/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef TAVIATABBUTTON_H
#define TAVIATABBUTTON_H

#include <qlabel.h>
#include <qiconset.h>
#include <qpixmap.h>
#include <qlayout.h>
#include <qfont.h>
#include <qpoint.h>
#include <qdatetime.h>
#include <qtoolbutton.h>
#include <qdragobject.h>
#include <qmap.h>
#include <ksqueezedtextlabel.h>
#include <taviatabbox.h>
#include <kurldrag.h>

/**
@author TAM(Teppei Tamra)
*/
class TaviaTabBox;

class TaviaTabMiniButton : public QLabel
{
friend class TaviaTabButton;

Q_OBJECT
public:
    TaviaTabMiniButton(QWidget *parent = 0, const char *name = 0, WFlags f = 0);

    ~TaviaTabMiniButton();

    void setIconSet(const QIconSet &i);

protected:
    QIconSet icon;
    bool press;
    virtual void mouseReleaseEvent(QMouseEvent *e);
    virtual void mousePressEvent(QMouseEvent *e);

public slots: 
    virtual void setText(const QString &text);

signals:
    void clicked();
};


class TaviaTabButtonConfig {
public:
    QColor bgColor;
    QColor fgColor;
    QColor activeColor;
    QColor selectedColor;
    unsigned int edgeR;
    unsigned int grad;
    bool roundEdge;
};


class TaviaTabButton : public QWidget
{
Q_OBJECT
public:
    TaviaTabButton(TaviaTabBox * parent = 0, const char * name = 0, WFlags f = 0);

    ~TaviaTabButton();
    bool isSelected();
    void setSelected(bool s);
    void setSize(unsigned int s);
    static QPixmap drawEdgePixmap(QColor &first,QColor &last,unsigned int size,unsigned int grad,bool roundRect = true,int r = 2,QBitmap *shape = 0);

protected:
    void drawEdge(QColor &first,QColor &last,unsigned int size,unsigned int grad,bool roundRect = true,int r = 2,bool shape = false);
    virtual void mouseReleaseEvent(QMouseEvent *e);
    virtual void mousePressEvent(QMouseEvent *e);
    virtual void mouseMoveEvent(QMouseEvent *e);
    virtual void wheelEvent(QWheelEvent *e);
    void redrawButton();
    void updateThumbnail();
    virtual void dropEvent(QDropEvent *e);
    virtual void dragEnterEvent(QDragEnterEvent *e);
public:
    TaviaTabMiniButton * leftButton;
    TaviaTabMiniButton * rightButton;
protected:
    TaviaTabBox *tabbox;
    TaviaTabButtonConfig * config;
    QGridLayout * contentLay;
    QFrame * content;
    KSqueezedTextLabel * label;
    QLabel * thumbnail;
    bool active;
    bool selected;
    QWidget * parentWidget;
    QPoint parentPos;
    QPoint buttonPos;
    unsigned int size;
    QMap<QDateTime,QPixmap> animationQueue;
    QPixmap thumbnailPix;
    QPixmap cacheThumbnailPix;
    bool press;
    bool close;
    KURL currentURL;
protected slots:
    void hoverCheck();
    void zoomIn();
    void zoomOut();
    void closeButtonClicked();
    void iconButtonClicked();
public slots:
    void setButtonText(const QString &text);
    void setThumbnail(const QPixmap &pix);
    void setURL(const KURL &url);
signals:
    void closeButtonClicked(TaviaTabButton *);
    void iconButtonClicked(TaviaTabButton *);
    void clicked(TaviaTabButton *);
    void dropURLList(KURL::List urllist);
};



#endif
