/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "taviaurcdrag.h"
#include <qcstring.h>
#include <qdatastream.h>
#include <kdebug.h>
#include <kiconloader.h>
#include <kglobal.h>

TaviaURCDrag::TaviaURCDrag(TaviaURCManager *m,QWidget *dragSource, const char *name)
 : QStoredDrag("application/x-tavia-urc-connect",dragSource, name)
{
    manager = m;
    setPixmap(KGlobal::instance()->iconLoader()->loadIcon("connect_creating",KIcon::Panel,KIcon::SizeMedium));
}

TaviaURCDrag::TaviaURCDrag(TaviaURCManager *m, TaviaURCObject *from, TaviaURCObject *old, QWidget *dragSource, const char *name)
 : QStoredDrag("application/x-tavia-urc-connect",dragSource, name)
{
    manager = m;
    setEncodedData(dataEncode(from,old));
    setPixmap(KGlobal::instance()->iconLoader()->loadIcon("connect_creating",KIcon::Panel,KIcon::SizeMedium));
}


TaviaURCDrag::~TaviaURCDrag()
{
}



/*!
    \fn TaviaURCDrag::canDecode(const QMimeSource *e)
 */
bool TaviaURCDrag::canDecode(const QMimeSource *e)
{
    return(e->provides("application/x-tavia-urc-connect"));
}


/*!
    \fn TaviaURCDrag::dataEncode(TaviaURCObject *from,TaviaURCObject *old)
 */
QByteArray TaviaURCDrag::dataEncode(TaviaURCObject *from,TaviaURCObject *old)
{
    QCString b;
    b = from->getId().ascii();
    b += ":";
    if (old) b += old->getId().ascii();
    kdDebug() << "DnD Data Encoded:" << b << endl;
    return(b);
}



/*!
    \fn TaviaURCDrag::encodedData(const char *mime) const
QByteArray TaviaURCDrag::encodedData(const char *mime) const
{
    QCString d;
    if (qstrnicmp(mime,"application/x-tavia-urc-connect",31) == 0) {
        d = mime + 31;
    }
    kdDebug() << "DnD Data :" << d << endl;
    return(d);
        
}

 */



/*!
    \fn TaviaURCDrag::decode(const QMimeSource *e,TaviaURCManager *m,TaviaURCObject *from,TaviaURCObject *old)
 */
bool TaviaURCDrag::decode(const QMimeSource *e,TaviaURCManager *m,TaviaURCObject *&from,TaviaURCObject *&old)
{
    QString d(e->encodedData("application/x-tavia-urc-connect"));
    QStringList s = QStringList::split(":",d,true);
    kdDebug() << "DropData: " << s[0] << " " << s[1] << endl;
    QPtrList<TaviaURCObject> p = m->getURCObjectList();
    from = 0L;
    old = 0L;
    kdDebug() << "URCCount " << p.count() << endl;
    for(unsigned int i = 0;i < p.count();i ++) {
        if (p.at(i)->getId() == s[0]) {
            from = p.at(i);
            break;
        }
    }
    if (s[1] == QString::null) {
        return(true);
    }
    for(unsigned int i = 0;i < p.count();i ++) {
        if (p.at(i)->getId() == s[1]) {
            old = p.at(i);
            break;
        }
    }
    kdDebug() << "URCObject - from : " << from << " old : " << old << endl;
    return(true);
}
