/***************************************************************************
 *   Copyright (C) 2003 by TAM(Teppei Tamra)                               *
 *   tam-t@par.odn.ne.jp                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "taviaviewmanager.h"
#include "taviapartsview.h"
#include <kxmlguiclient.h>
#include <kxmlguifactory.h>
#include <qmap.h>

#include <qiconset.h>

TaviaViewManager::TaviaViewManager(TaviaTabBox *_tabbox, TaviaMdiFrm *_mdi ,QObject *parent, const char *name)
 : QObject(parent, name) ,KXMLGUIClient()
{
    closeProcess = false;
    tabbox = _tabbox;
    mdi = _mdi;
    currentView = 0L;
    partManager = new KParts::PartManager(mdi);//static_cast<QWidget *>(parent));
    urcManager = new TaviaURCManager(parent);
    connect(partManager,SIGNAL(activePartChanged(KParts::Part *)),this,SIGNAL(activePart(KParts::Part* )));
    
    setXMLFile("taviaviewmanagerui.rc");

    
    new KAction(i18n("Create &New View"),"tab_new",CTRL+Key_N,this,SLOT(addPartsView()),actionCollection(),"addNewView");

}


TaviaViewManager::~TaviaViewManager()
{
    //for(unsigned int i = 0;i < viewList.count();i ++) {
    //    delView(viewList.at(i));
    //}
}



/*!
    \fn TaviaViewManager::entrySearch(TaviaTabButton *e)
 */
TaviaViewEntry * TaviaViewManager::entrySearch(TaviaTabButton *e)
{
    for(unsigned int i = 0;i < viewList.count();i ++) {
        if (viewList.at(i)->button == e) return viewList.at(i);
    }
    return 0L;
}


/*!
    \fn TaviaViewManager::entrySearch(TaviaView *e)
 */
TaviaViewEntry * TaviaViewManager::entrySearch(TaviaView *e)
{
    for(unsigned int i = 0;i < viewList.count();i ++) {
        if (viewList.at(i)->view == e) return viewList.at(i);
    }
    return 0L;
}


/*!
    \fn TaviaViewManager::entrySearch(KMdiChildView *e)
 */
TaviaViewEntry * TaviaViewManager::entrySearch(KMdiChildView *e)
{
    for(unsigned int i = 0;i < viewList.count();i ++) {
        if (viewList.at(i)->childView == e) return viewList.at(i);
    }
    return 0L;
}


/*!
    \fn TaviaViewManager::getTabBox()
 */
TaviaTabBox * TaviaViewManager::getTabBox()
{
    return(tabbox);
}


/*!
    \fn TaviaViewManager::getMdiFrm()
 */
TaviaMdiFrm * TaviaViewManager::getMdiFrm()
{
    return(mdi);
}


/*!
    \fn TaviaViewManager::getPartManager()
 */
KParts::PartManager * TaviaViewManager::getPartManager()
{
    return(partManager);
}


/*!
    \fn TaviaViewManager::getURCManager()
 */
TaviaURCManager * TaviaViewManager::getURCManager()
{
    return(urcManager);
}


/*!
    \fn TaviaViewManager::updateView()
 */
void TaviaViewManager::updateView()
{
    tabbox->updateBox();
}


/*!
    \fn TaviaViewManager::addView(TaviaView *view)
 */
TaviaViewEntry * TaviaViewManager::addView(TaviaView *view)
{
    connect(view,SIGNAL(setViewCaption(const QString& )),this,SLOT(setTitleText(const QString& )));
    connect(view,SIGNAL(setViewStatusText(const QString& )),this,SLOT(setStatusText(const QString& )));
    connect(view,SIGNAL(setViewIcon(const QPixmap &)),this,SLOT(setIcon(const QPixmap& )));
    connect(view,SIGNAL(closeView(TaviaView* )),this,SLOT(closeView(TaviaView* )));
    TaviaViewEntry *entry = new TaviaViewEntry;
    //TaviaTabButton *button = new TaviaTabButton(tabbox);
    entry->view = view;
    entry->button = tabbox->addButton();
    connect(entry->button,SIGNAL(closeButtonClicked(TaviaTabButton* )),this,SLOT(closeView(TaviaTabButton* )));
    connect(entry->button,SIGNAL(clicked(TaviaTabButton* )),this,SLOT(setSelected(TaviaTabButton* )));
    connect(entry->button,SIGNAL(dropURLList(KURL::List )),entry->view,SLOT(openURLList(KURL::List)));
    connect(view,SIGNAL(currentURL(const KURL& )),entry->button,SLOT(setURL(const KURL& )));
    connect(view,SIGNAL(setViewCaption(const QString& )),entry->button,SLOT(setButtonText(const QString& )));
    connect(view,SIGNAL(thumbnailReady(const QPixmap& )),entry->button,SLOT(setThumbnail(const QPixmap& )));
    viewList.append(entry);
    entry->childView = mdi->createWrapper(view,"view",0);
    // ĲΤ줬פ롩
    entry->childView->resize(mdi->defaultChildFrmSize());
    mdi->addWindow(entry->childView);
    connect(entry->childView,SIGNAL(activated (KMdiChildView *)),this,SLOT(setSelected(KMdiChildView* )));
    connect(entry->childView,SIGNAL(childWindowCloseRequest (KMdiChildView *)),this,SLOT(closeView(KMdiChildView* )));
    

    updateView();
    return(entry);
}




/*!
    \fn TaviaViewManager::delView(TaviaViewEntry *entry)
 */
void TaviaViewManager::delView(TaviaViewEntry *entry)
{
    kdDebug() << "delView : " << entry << endl;
    //if (viewList.find(entry) != -1) {
    if (viewList.remove(entry)) {
        // ʲ֤˥ӥǤΤդ٤
        //viewList.remove(viewList.find(entry));
        if (currentView == entry) {
            // ̯
            activePart(0L);
            currentView = 0L;
            //if (viewList.count()) setSelected(viewList.count() - 1);
        }
        if (entry->button) tabbox->delButton(entry->button);
        if (entry->view) {
            disconnect(entry->view,SIGNAL(closeView(TaviaView* )),this,SLOT(closeView(TaviaView* )));
            delete entry->view;
        }
        if (entry->childView) {
            disconnect(entry->childView,SIGNAL(childWindowCloseRequest (KMdiChildView *)),this,SLOT(closeView(KMdiChildView* )));
            mdi->closeWindow(entry->childView);
            // @todo mdi->closeWindowchildViewdelete뤫ǧ
            // delete entry->childView;
        }
        //delete entry;
    }
}

/*!
    \fn TaviaViewManager::addPartsView(const KURL &url,const KParts::URLArgs &args)
 */
void TaviaViewManager::addPartsView(const KURL &url,const KParts::URLArgs &args)
{
    KParts::ReadOnlyPart *p = 0;
    return(addPartsView(url,args,KParts::WindowArgs(),p));
}

/*!
    \fn TaviaViewManager::addPartsView(const KURL &url,const KParts::URLArgs &args)
 */
void TaviaViewManager::addPartsView(const KURL &url,const KParts::URLArgs &args,const KParts::WindowArgs &wargs,KParts::ReadOnlyPart *&p)
{
    kdDebug() << "AddPartsView URL: " << url << endl;
    TaviaPartsView *v = new TaviaPartsView(this,mdi);
    connect(v,SIGNAL(activePart(KParts::Part* )),this,SIGNAL(activePart(KParts::Part* )));
    addView(v);
    KParts::URLArgs a = args;
    if (!a.metaData().contains("_TAVIA_VIEW_BG")) setSelected(v);
    p = v->part;
    if (!url.isEmpty()) v->openURL(url,args);
}


/*!
    \fn TaviaViewManager::setTitleText(const QString &title)
 */
void TaviaViewManager::setTitleText(const QString &title)
{
    // إ()
    //static_cast<KMdiChildView *>(static_cast<TaviaView *>(const_cast<QObject *>(sender()))->parent())->mdiParent()->setCaption(title);
    TaviaViewEntry *e = entrySearch(static_cast<TaviaView *>(const_cast<QObject *>(sender())));
    e->childView->setCaption(title);
    e->caption = title;
    
}


/*!
    \fn TaviaViewManager::setStatusText(const QString &text)
 */
void TaviaViewManager::setStatusText(const QString &text)
{
    /// @todo implement me
    kdDebug() << "StatusText:" << text << endl;
}


/*!
    \fn TaviaViewManager::setIcon(const QPixmap &icon)
 */
void TaviaViewManager::setIcon(const QPixmap &icon)
{
    TaviaViewEntry *e = entrySearch(static_cast<TaviaView *>(const_cast<QObject *>(sender())));
    KMdiChildFrm *f = e->childView->mdiParent();
    if (f) f->setIcon(icon);
    e->childView->setIcon(icon);
    e->button->leftButton->setIconSet((QIconSet(icon)));
}


/*!
    \fn TaviaViewManager::setSelected(TaviaTabButton *b)
 */
void TaviaViewManager::setSelected(TaviaTabButton *b)
{
    setSelected(entrySearch(b));
}


/*!
    \fn TaviaViewManager::setSelected(TaviaView *v)
 */
void TaviaViewManager::setSelected(TaviaView *v)
{
    setSelected(entrySearch(v));
}


/*!
    \fn TaviaViewManager::setSelected(unsigned int s)
 */
void TaviaViewManager::setSelected(unsigned int s)
{
    setSelected(viewList.at(s));
}

/*!
    \fn TaviaViewManager::setSelected(KMdiChildView *cv)
 */
void TaviaViewManager::setSelected(KMdiChildView *cv)
{
    if (currentView) {
        if (currentView->childView == cv) return;
    }
    if (!(entrySearch(cv)->button->isSelected())) {
        setSelected(entrySearch(cv));
    }
}


/*!
    \fn TaviaViewManager::setSelected(const TaviaViewEntry *e)
 */
void TaviaViewManager::setSelected(TaviaViewEntry *e)
{
    if (!e) return;
    if ((!viewList.count())) {
        currentView = 0L;
        return;
    }
    if ((!e->button) || (!e->view) || (!e->childView)) return;
    
    for(unsigned int i = 0;i < viewList.count();i ++) {
        if (viewList.at(i) == e) {
            e->button->setSelected(true);
            kdDebug() << "view selected: " << i << endl;
        } else {
            viewList.at(i)->button->setSelected(false);
        }
    }
    //emit tabbox->updateBox();
    if (currentView) {
        if (viewList.find(currentView) != -1) 
            if (currentView->view) currentView->view->deactivate();
    }
    currentView = e;
    if (e->childView->isAttached()) e->childView->mdiParent()->raiseAndActivate();
    else e->childView->activate();
    e->view->activate();
    
    mdi->setCaption(e->caption);
}








/*!
    \fn TaviaViewManager::closeView(KMdiChildView *cv)
 */
void TaviaViewManager::closeView(KMdiChildView *cv)
{
    TaviaViewEntry * e = entrySearch(cv);
    if (!e) return;
    disconnect(e->view,SIGNAL(closeView(TaviaView* )),this,SLOT(closeView(TaviaView* )));
    e->childView = 0L;
    delView(e);
}


/*!
    \fn TaviaViewManager::closeView(TaviaView *v)
 */
void TaviaViewManager::closeView(TaviaView *v)
{
    TaviaViewEntry * e = entrySearch(v);
    if (!e) return;
    disconnect(e->childView,SIGNAL(childWindowCloseRequest (KMdiChildView *)),this,SLOT(closeView(KMdiChildView* )));
    e->view = 0L;
    delView(e);
}


/*!
    \fn TaviaViewManager::closeView(KMdiChildView *cv)
 */
void TaviaViewManager::closeView(TaviaTabButton *b)
{
    TaviaViewEntry * e = entrySearch(b);
    if (!e) return;
    e->button = 0L;
    tabbox->delButton(b);
    disconnect(e->view,SIGNAL(closeView(TaviaView* )),this,SLOT(closeView(TaviaView* )));
    disconnect(e->childView,SIGNAL(childWindowCloseRequest (KMdiChildView *)),this,SLOT(closeView(KMdiChildView* )));
    delView(e);
}




/*!
    \fn TaviaViewManager::isSelected(TaviaView *view)
 */
bool TaviaViewManager::isSelected(TaviaView *view)
{
    /* int s = viewList.find(entrySearch(view));
    if (s < 0) return(false);
    if (viewList.at(s)->view == view) return(true);
    return(false); */
    if (entrySearch(view) == currentView) return(true);
    else return(false);
}


/*!
    \fn TaviaViewManager::openURLwithCurrentView(const KURL &url,const KParts::URLArgs &args)
 */
void TaviaViewManager::openURLwithCurrentView(const KURL &url,const KParts::URLArgs &args)
{
    if (!viewList.count()) addPartsView(url,args);
    else currentView->view->openURL(url,args);
}
